# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class AviraParser(KaitaiStruct):
    """Creator: Florian Bausch, ERNW Research GmbH, https://ernw-research.de
    License: CC-BY-SA-4.0 https://creativecommons.org/licenses/by-sa/4.0/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.magic = self._io.read_bytes(16)
        if not self.magic == b"\x41\x6E\x74\x69\x56\x69\x72\x20\x51\x75\x61\x00\x00\x00\x00\x00":
            raise kaitaistruct.ValidationNotEqualError(b"\x41\x6E\x74\x69\x56\x69\x72\x20\x51\x75\x61\x00\x00\x00\x00\x00", self.magic, self._io, u"/seq/0")
        self.malicious_offset = self._io.read_u4le()
        self.len_filename = self._io.read_u4le()
        self.len_addl_info = self._io.read_u4le()
        self.unknown1 = self._io.read_bytes(32)
        self.qua_time = self._io.read_u4le()
        self.unknown2 = self._io.read_bytes(92)
        self.mal_type = (KaitaiStream.bytes_terminate(self._io.read_bytes(64), 0, False)).decode(u"UTF-8")
        self.filename = (self._io.read_bytes((self.len_filename if self.len_filename < 2 else (self.len_filename - 2)))).decode(u"UTF-16LE")
        if self.len_filename >= 2:
            self.padding1 = self._io.read_bytes(2)
            if not self.padding1 == b"\x00\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00\x00", self.padding1, self._io, u"/seq/9")

        self.addl_info = (self._io.read_bytes((self.len_addl_info if self.len_addl_info < 2 else (self.len_addl_info - 2)))).decode(u"UTF-16LE")
        if self.len_addl_info >= 2:
            self.padding2 = self._io.read_bytes(2)
            if not self.padding2 == b"\x00\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00\x00", self.padding2, self._io, u"/seq/11")

        self._raw_mal_file = self._io.read_bytes_full()
        self.mal_file = KaitaiStream.process_xor_one(self._raw_mal_file, 170)





from .quarantine import *
from maldump.parsers.forticlient_parser import ForticlientParser


class FortiClient(Quarantine):
    """Implements FortiClient quarantine format"""

    def __init__(self):
        super().__init__()
        self.name = 'FortiClient'
        self.location = Path('Program Files/Fortinet/FortiClient/quarantine')

    def _normalize_path(self, path):
        if path[2:4] == '?\\':
            path = path[4:]
        return path

    def _get_time(self, ts):
        return dt(ts.year, ts.month, ts.day, ts.hour, ts.minute, ts.second)

    def export(self):
        quarfiles = []
        for metafile in self.location.glob('*[!.meta]'):
            kt = ForticlientParser.from_file(metafile)
            q = QuarEntry()
            q.timestamp = self._get_time(kt.timestamp)
            q.threat = kt.mal_type
            q.path = self._normalize_path(kt.mal_path)
            q.size = kt.mal_len
            q.md5 = md5(kt.mal_file).digest().hex()
            q.malfile = kt.mal_file
            quarfiles.append(q)
            kt.close()

        return quarfiles



from .quarantine import *
from maldump.parsers.windef_entries import WindefEntries
from maldump.parsers.windef_resource_data import WindefResourceData


class WindowsDefender(Quarantine):
    """Implements Windows Defender quarantine format"""

    def __init__(self):
        super().__init__()
        self.name = 'Microsoft Defender'
        self.location = Path('ProgramData/Microsoft/Windows Defender/Quarantine')

    def _normalize(self, path_chrs):
        path_str = ''.join(map(chr, path_chrs[:-1]))
        if path_str[2:4] == '?\\':
            path_str = path_str[4:]
        return path_str

    def _get_malfile(self, guid):
        quarfile = self.location / 'ResourceData' / guid[:2] / guid
        kt = WindefResourceData.from_file(quarfile)
        malfile = kt.encryptedfile.mal_file
        kt.close()
        return malfile

    def export(self):
        quarfiles = []
        for metafile in self.location.glob('Entries/{*}'):
            kt = WindefEntries.from_file(metafile)
            ts = dt.fromtimestamp(int(kt.data1.time.unixts))

            # Loop through all entries, if they exist
            for e in kt.data2.entries:
                # Support only 'file' type for now
                if e.entry.typestr == 'file':
                    guid = e.entry.element[0].content.value.hex().upper()
                    malfile = self._get_malfile(guid)
                    q = QuarEntry()
                    q.timestamp = ts
                    q.threat = kt.data1.mal_type
                    q.path = self._normalize(e.entry.path.character)
                    q.size = len(malfile)
                    q.md5 = md5(malfile).digest().hex()
                    q.malfile = malfile
                    quarfiles.append(q)
            kt.close()

        return quarfiles



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import maldump.utils as utils


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class GdataParser(KaitaiStruct):
    """Creator: Florian Bausch, ERNW Research GmbH, https://ernw-research.de
    License: CC-BY-SA-4.0 https://creativecommons.org/licenses/by-sa/4.0/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.magic1 = self._io.read_bytes(4)
        if not self.magic1 == b"\xCA\xFE\xBA\xBE":
            raise kaitaistruct.ValidationNotEqualError(b"\xCA\xFE\xBA\xBE", self.magic1, self._io, u"/seq/0")
        self.len_data1 = self._io.read_u4le()
        self._raw__raw_data1 = self._io.read_bytes(self.len_data1)
        _process = utils.CustomArc4(b"\xA7\xBF\x73\xA0\x9F\x03\xD3\x11\x85\x6F\x00\x80\xAD\xA9\x6E\x9B")
        self._raw_data1 = _process.decode(self._raw__raw_data1)
        _io__raw_data1 = KaitaiStream(BytesIO(self._raw_data1))
        self.data1 = GdataParser.EncryptedData1(_io__raw_data1, self, self._root)
        self.magic2 = self._io.read_bytes(4)
        if not self.magic2 == b"\xBA\xAD\xF0\x0D":
            raise kaitaistruct.ValidationNotEqualError(b"\xBA\xAD\xF0\x0D", self.magic2, self._io, u"/seq/3")
        self.len_data2 = self._io.read_u4le()
        self._raw__raw_data2 = self._io.read_bytes(self.len_data2)
        _process = utils.CustomArc4(b"\xA7\xBF\x73\xA0\x9F\x03\xD3\x11\x85\x6F\x00\x80\xAD\xA9\x6E\x9B")
        self._raw_data2 = _process.decode(self._raw__raw_data2)
        _io__raw_data2 = KaitaiStream(BytesIO(self._raw_data2))
        self.data2 = GdataParser.EncryptedData2(_io__raw_data2, self, self._root)
        self._raw_mal_file = self._io.read_bytes_full()
        _process = utils.CustomArc4(b"\xA7\xBF\x73\xA0\x9F\x03\xD3\x11\x85\x6F\x00\x80\xAD\xA9\x6E\x9B")
        self.mal_file = _process.decode(self._raw_mal_file)

    class EncryptedData1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unknown1 = self._io.read_u4le()
            self.unknown2 = self._io.read_u4le()
            self.unknown3 = self._io.read_u4le()
            self.quatime = self._io.read_u4le()
            self.unknown5 = self._io.read_u4le()
            self.malwaretype = GdataParser.Utf16le(self._io, self, self._root)


    class EncryptedData2(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unknown1 = self._io.read_u4le()
            self.unknown2 = self._io.read_u4le()
            self.filesize = self._io.read_u4le()
            self.unknownstring1 = GdataParser.Utf16le(self._io, self, self._root)
            self.unknown4 = self._io.read_u4le()
            self.unkown5 = self._io.read_u4le()
            self.time1 = GdataParser.Winfiletime(self._io, self, self._root)
            self.time2 = GdataParser.Winfiletime(self._io, self, self._root)
            self.time3 = GdataParser.Winfiletime(self._io, self, self._root)
            self.unknown6 = self._io.read_u4le()
            self.filesize2 = self._io.read_u4le()
            self.path = GdataParser.Utf16le(self._io, self, self._root)


    class Utf16le(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bom = self._io.read_bytes(3)
            if not self.bom == b"\xFF\xFE\xFF":
                raise kaitaistruct.ValidationNotEqualError(b"\xFF\xFE\xFF", self.bom, self._io, u"/types/utf16le/seq/0")
            self.number_of_chars = self._io.read_u1()
            self.string_content = (self._io.read_bytes((self.number_of_chars * 2))).decode(u"utf-16le")


    class Winfiletime(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ts = self._io.read_u8le()

        @property
        def unixts(self):
            if hasattr(self, '_m_unixts'):
                return self._m_unixts if hasattr(self, '_m_unixts') else None

            self._m_unixts = ((self.ts * 1E-7) - 11644473600)
            return self._m_unixts if hasattr(self, '_m_unixts') else None






import json

from .quarantine import *
from maldump.utils import CustomArc4


class Malwarebytes(Quarantine):
    """Implements Malwarebytes quarantine format"""

    def __init__(self):
        super().__init__()
        self.name = 'Malwarebytes'
        self.location = Path('ProgramData/Malwarebytes/MBAMService/Quarantine')

    def _decrypt(self, input):
        key = [
            0x03, 0x7a, 0x55, 0xc5,
            0xdf, 0x39, 0xd2, 0x89,
            0x7f, 0xef, 0x88, 0x3d,
            0xc0, 0x47, 0xb3, 0x17
        ]
        return CustomArc4(key).decode(input)

    def _normalize_time(self, date):
        return dt.strptime(date, '%Y-%m-%dT%H:%M:%SZ')

    def export(self):
        quarfiles = []
        for metafile in self.location.glob('*.data'):
            with open(metafile, 'rb') as f:
                decrypted = self._decrypt(f.read())
            metadata = json.loads(decrypted)

            # Open corresponding malfile
            uid = metadata['trace']['objectID']
            with open(self.location / (uid + '.quar'), 'rb') as f:
                malfile = self._decrypt(f.read())

            q = QuarEntry()
            q.timestamp = self._normalize_time(metadata['trace']['cleanTime'])
            q.threat = metadata['threatName']
            q.path = metadata['trace']['objectPath']
            q.size = len(malfile)
            q.md5 = metadata['trace']['objectMD5'].lower()
            q.malfile = malfile
            quarfiles.append(q)

        return quarfiles



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class ForticlientParser(KaitaiStruct):
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.magic = self._io.read_bytes(8)
        if not self.magic == b"\x51\x55\x41\x52\x46\x00\x00\x00":
            raise kaitaistruct.ValidationNotEqualError(b"\x51\x55\x41\x52\x46\x00\x00\x00", self.magic, self._io, u"/seq/0")
        self.unknown1 = self._io.read_u4le()
        self.mal_offset = self._io.read_u4le()
        self.unknown2 = self._io.read_bytes(36)
        self.mal_len = self._io.read_u4le()
        self._raw_timestamp = self._io.read_bytes(16)
        _io__raw_timestamp = KaitaiStream(BytesIO(self._raw_timestamp))
        self.timestamp = ForticlientParser.Timestamp(_io__raw_timestamp, self, self._root)
        self.unknown3 = self._io.read_bytes(12)
        self.file_id = self._io.read_u4le()
        self.len_mal_path = self._io.read_u4le()
        self.len_mal_type = self._io.read_u4le()
        self.mal_path = (self._io.read_bytes(self.len_mal_path)).decode(u"UTF-16LE")
        self.mal_type = (self._io.read_bytes(self.len_mal_type)).decode(u"UTF-16LE")
        self._raw_mal_file = self._io.read_bytes_full()
        self.mal_file = KaitaiStream.process_xor_one(self._raw_mal_file, 171)

    class Timestamp(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.year = self._io.read_u2le()
            self.month = self._io.read_u2le()
            self.tz_offset = self._io.read_u2le()
            self.day = self._io.read_u2le()
            self.hour = self._io.read_u2le()
            self.minute = self._io.read_u2le()
            self.second = self._io.read_u2le()
            self.microsecond = self._io.read_u2le()






"""
    Convenience utils for use in avs and parsers
"""

from Crypto.Cipher import ARC4


def xor(input, key):
    result = bytearray(input)
    key_len = len(key)
    for i in range(len(input)):
        result[i] ^= key[i % key_len]
    return bytes(result)


class CustomArc4(object):
    def __init__(self, key):
        self.key = bytes(key)

    def decode(self, plaintext):
        cipher = ARC4.new(self.key)
        return cipher.decrypt(plaintext)
